function y = bezier_reconstruction(problem,t,display)
% Returns the value of the Bezier spline controlled by control points cp
% at time t.
%
% function y = bezier_reconstruction(problem,t,method)
%    returns y, a matrix of size [m,n,p] containing the value of the
%    hybrid Bezier spline B at times t. B is composed of second order 
%    Bezier functions as first and last segments, and cubic Bezier 
%    functions as intermediate segments. t may be a vector of values
%    with 0 <= t <= (size(cp) + 2)/3
% 
% Input: problem, the structure of the problem with 
% 			control : control points on M
% 		 t, the time at which the curve is evaluated
% 		 display, if a waitbar must be displayed
%                  
% Output: y, the value of the spline at t, stored in a [m x n x p].
%
% Original author: 
%   Pierre-Yves Gousenbourger, Jan. 01, 2013
% Contributors: 
%
% Change log:
% 	Jan. 12, 2013 (PYG): 
% 		first version.
% 	Mar. 21, 2013 (PYG): 
% 		final version for master thesis.
% 	Feb. 30, 2014 (PYG):
% 		refactor for new structure.
% 	Mar. 03, 2016 (PYG):
% 		acceleration of the method.
% 	Mar. 14, 2018 (PYG): 
% 		Manopt integration and alignment with fitting methods.
    
    
    if nargin < 3
		display = 'true';
	end
	wb = strcmp(display,'true');
		
	% Data
	cp 		= problem.control;				% computed control points
	M  		= problem.M;					% manifold
	[l,c,m] = size(cp);                     % (l,c), size of the space
											% m, number of cp
	n 	  	= (m + 4)/3;					% number of data points
	p 		= length(t);                    % number of computed points
	f = @(x,ti) decasteljau(M,x,ti);		% method for reconstruction
  
  
  
	% Waitbar because it is fun...
	if wb; h = waitbar(0,'Reconstruction. Please wait...');  total = p; growth = 0; end
	
	
	% Preallocation
	y 		= zeros(l,c,p);
		


	% Compute the Bezier spline.
	% In the case of only two control points, only a geodesic is needed.
	% In the other cases, we apply f.
	switch m
	case 2
		a = cp(:,:,1);
		b = cp(:,:,2);
		for i = 1:p
			y(:,:,i) = M.exp(a,M.log(a,b),t(i));
			if wb; growth = growth + 1;	 waitbar(growth/total,h,sprintf('Reconstruction. Complete: %.f %%.',growth/total*100)); end
		end

	otherwise	
		for i = 1:p
			ti   = t(i);
			sgmt = floor(ti)+1;
			
			if sgmt == n ; y(:,:,i) = cp(:,:,3*n-4); % last point
			elseif ti == 0; y(:,:,1) = cp(:,:,1);
			elseif sgmt == ti+1 ; y(:,:,i) = cp(:,:,3*(sgmt-1)); % first or inner points
			else
				switch sgmt
					case 1;    range = [1:3];
					case n-1;  range = [m-2:m];
					otherwise; range = [3*(sgmt-1):3*sgmt];
				end
				
				ti = ti - sgmt + 1;
				
				y(:,:,i) = f(cp(:,:,range),ti);
			end
			if wb; growth = growth + 1;	 waitbar(growth/total,h,sprintf('Reconstruction. Complete: %.f %%.',growth/total*100)); end
		end
	end
	
	% End of waitbar
	if wb; close(h); end
end
