function y = bezierlike_one(problem,t,display)
% Returns the value of the Bezier spline controlled by control points cp
% at time t.
%
% function y = bezierlike_one(problem,t)
%    returns y, a matrix of size [m,n,p] containing the value of the
%    cubic Bezier spline B at times t.
% 
% Input: problem, the structure of the problem with 
% 			control : control points on M
% 			Tcp 	: control points on the tangent spaces of di
% 			data 	: the data points
% 		 t, the time at which the curve is evaluated
% 		 display, if a waitbar must be displayed
%                  
% Output: y, the value of the spline at t, stored in a [m x n x p].
%
% Original author: 
%   Pierre-Yves Gousenbourger, Oct. 19, 2016
% Contributors: 
%
% Change log:
% 	Oct. 19, 2016 (PYG):
%      First version
% 	Dec. 14, 2017 (PYG):
% 	   Modification for cubic splines
% 	Jan. 24, 2018 (PYG):
% 	   Modification of the reconstruction process. The first step is now
% 	   an Exp mapping of the straight line linking b_i^+, p_i and b_i^-
% 	   in the tangent space of d_i. Therefore, the curve is no more a
% 	   Bezier curve, but a Bezier-like curve.
% 	Feb. 27, 2018 (PYG):
% 	   Modification to the new storage of Tcp.
	
	
	if nargin < 3
		display = 'true';
	end
	wb = strcmp(display,'true');
	
	% Data
	
	d	 	  = problem.data;					% data points on M
	Tcp 	  = problem.Tcp;					% cp on T_di M
	cp 		  = problem.control;				% cp on M
	M  		  = problem.M;						% manifold
	[l,c,m,n] = size(Tcp);                     	% (l,c), size of the space
												% m, number of cp
												% n, number of data points
	p 		  = length(t);                    	% size of the discretization
	f 		  = @(x,ti) decasteljau(M,x,ti);	% reconstruction method
	
	
	% Waitbar because it is fun...
	if wb; h = waitbar(0,'Reconstruction. Please wait...'); total = p; growth = 0; end
	


	% Reconstruction of the curve
	y 		= zeros(l,c,p);
	
	switch m
	case 2		% 2 cp: geodesic
		a = cp(:,:,1);  b = cp(:,:,2);
		
		for i = 1:p 
			y(:,:,i) = M.exp(a,M.log(a,b),t(i));
			
			% Waitbar
			if wb
				growth = growth + 1; 
				waitbar(growth/total,h,sprintf('Reconstruction. Complete: %.f %%.',growth/total*100));
			end
		end

	otherwise 	% more than 2 cp : two steps
				% 	1- Compute the straight line between Tb_i^- and 
				% 	   Tb_i^+, in T_di M. By construction, p_i is in the
				% 	   middle. Map the line to M.
				% 	2- Continue de Casteljau normally.
				
	
		for i = 1:p
			ti   = t(i);
			sgmt = floor(ti)+1;
			
			if sgmt == n ; y(:,:,i) = cp(:,:,3*n-2); % last point
			elseif sgmt == ti+1 ; y(:,:,i) = cp(:,:,3*(sgmt-1)+1); % first or inner points
			else
				range = [3*(sgmt-1)+1:3*sgmt+1];
				ti = ti - sgmt + 1;
				
				Tb(:,:,1:2) = Tcp(:,:,range(1:2),sgmt  );	% Useful Tcp
				Tb(:,:,3:4) = Tcp(:,:,range(3:4),sgmt+1);
				
				b  =  cp(:,:,range(2:3));	    			% Useful cp
								
				% Step 1 : hybrid
				bb = zeros(l,c,3);
				
				bb(:,:,1) = M.exp(d(:,:,sgmt  ),  ti*Tb(:,:,2) + (1-ti)*Tb(:,:,1));    % Straight line between b1 and b2 in T_pj M
				bb(:,:,2) = M.exp(b(:,:,1)     ,  M.log(b(:,:,1),b(:,:,2)),ti); 	   % Geodesic between b2 and b3
				bb(:,:,3) = M.exp(d(:,:,sgmt+1),  ti*Tb(:,:,4) + (1-ti)*Tb(:,:,3));    % Straight line between b3 and b4 in T_p(j+1) M
				
				
				% Step 2 : classic de casteljau on bb
				y(:,:,i) = f(bb,ti);
			end
			if wb; growth = growth + 1;	 waitbar(growth/total,h,sprintf('Reconstruction. Complete: %.f %%.',growth/total*100)); end
		end
		
	end
	
	% End of waitbar
	if wb; close(h); end
end
