function y = bezierlike_semifull(problem,t,display)
% Returns the value of the Bezier spline controlled by control points cp
% at time t.
%
% function y = bezierlike_semifull(problem,t)
%    returns y, a matrix of size [m,n,p] containing the value of the
%    cubic Bezier spline B at times t.
% 
% Input: problem, the structure of the problem with 
% 			control : control points on M
% 			Tcp 	: control points on the tangent spaces of di
% 			data 	: the data points
% 		 t, the time at which the curve is evaluated
% 		 display, if a waitbar must be displayedp.
%                  
% Output: y, the value of the spline at t, stored in a [m x n x p].
%
% Original author: 
%   Pierre-Yves Gousenbourger, Oct. 19, 2016
% Contributors: 
%
% Change log:
% 	Oct. 19, 2016 (PYG):
%      First version
% 	Dec. 14, 2017 (PYG):
% 	   Modification for cubic splines
% 	Jan. 24, 2018 (PYG):
% 	   Modification of the reconstruction process. The first step is now
% 	   an Exp mapping of the straight line linking b_i^+, p_i and b_i^-
% 	   in the tangent space of d_i. Therefore, the curve is no more a
% 	   Bezier curve, but a Bezier-like curve.
% 	Fev. 27, 2018 (PYG) :
% 		Modification of the reconstruction process. The n-1 first steps
% 		of the reconstruction of the i^th segment are done in the 
% 		tangent spaces of p_i and p_{i+1}. Then, the last step is
% 		a classical geodesic.
	
	
	if nargin < 3
		display = 'true';
	end
	wb = strcmp(display,'true');
	
	% parameters
	
	d	 	  = problem.data;					% data points on M
	Tcp 	  = problem.Tcp;					% cp on T_di M
	cp 		  = problem.control;				% cp on M
	M  		  = problem.M;						% manifold
	[l,c,m,n] = size(Tcp);                     	% (l,c), size of the space
												% m, number of cp
												% n, number of data points
	p 		  = length(t);                    	% size of the discretization
	f 		  = @(x,ti) decasteljau_eucl(x,ti);	% reconstruction method
	
	
	% Waitbar because it is fun...
	if wb; h = waitbar(0,'Reconstruction. Please wait...'); total = p; growth = 0; end
	


	% Reconstruction of the curve
	y 		= zeros(l,c,p);
	
	
	switch m
	case 2		% 2 cp: geodesic
		a = cp(:,:,1);  b = cp(:,:,2);
		
		for i = 1:p 
			y(:,:,i) = M.exp(a,M.log(a,b),t(i));
			
			% Waitbar
			if wb
				growth = growth + 1; 
				waitbar(growth/total,h,sprintf('Reconstruction. Complete: %.f %%.',growth/total*100));
			end
		end



	% more than 2 cp : three steps
	% 	1- Compute the Bezier spline on T pi M and T p(i+1) M for the d-1 first cp.
	% 	2- Map the result to M.
	% 	3- Do the last step on M.
	otherwise		
		x = zeros(l,c,2);
		
		% First, fill Tcp with missing points.
		for j = 1:n-1
			Tcp(:,:,3*j,j)     = M.log(d(:,:,j),cp(:,:,3*j));
			Tcp(:,:,3*j-1,j+1) = M.log(d(:,:,j+1),cp(:,:,3*j-1));
		end
		
		% Then, reconstruction
		for i = 1:p
			ti   = t(i);
			sgmt = floor(ti)+1;
			
			if sgmt == n ; y(:,:,i) = M.exp(d(:,:,n),Tcp(:,:,3*n-2,n)); % last point
			elseif sgmt == ti+1 ; y(:,:,i) = M.exp(d(:,:,sgmt),Tcp(:,:,3*(sgmt-1)+1,sgmt)); % first or inner points
			else
				range = [3*(sgmt-1)+1:3*sgmt+1];
				
				% De casteljau on the two tangent spaces
				ti = ti - sgmt + 1;
				a = M.exp(d(:,:,sgmt)  ,f(Tcp(:,:,range(1:3),sgmt  ),ti));
				b = M.exp(d(:,:,sgmt+1),f(Tcp(:,:,range(2:4),sgmt+1),ti));
				
				
				% Last step : normal geodesic
				y(:,:,i) = M.exp(a,M.log(a,b),ti);
				
			end
			if wb; growth = growth + 1;	 waitbar(growth/total,h,sprintf('Reconstruction. Complete: %.f %%.',growth/total*100)); end
		end
	end
	
	% End of waitbar
	if wb; close(h); end
end
